@@ -34,6 +34,7 @@ typedef struct _PgmEventKey          PgmEventKey;
 typedef struct _PgmEventExpose       PgmEventExpose;
 typedef struct _PgmEventConfigure    PgmEventConfigure;
 typedef struct _PgmEventDnd          PgmEventDnd;
+typedef struct _PgmEventState        PgmEventState;
 typedef struct _PgmEventWin32Message PgmEventWin32Message;
 typedef union  _PgmEvent             PgmEvent;
 
@@ -65,6 +66,7 @@ G_BEGIN_DECLS
  * @PGM_DRAG_DROP: the mouse has dropped data onto the viewport.
  * @PGM_DRAG_LEAVE: the mouse has left the viewport while a drag is in progress.
  * @PGM_SCROLL: the scroll wheel was turned.
+ * @PGM_STATE: the state of a viewport has changed.
  * @PGM_DELETE: the window manager has requested that the toplevel window be
  * destroyed, usually when the user clicks on a special icon in the title bar.
  * @PGM_WIN32_MESSAGE: a Win32 message has been received.
@@ -87,8 +89,9 @@ typedef enum {
   PGM_DRAG_DROP           = 12,
   PGM_DRAG_LEAVE          = 13,
   PGM_SCROLL              = 14,
-  PGM_DELETE              = 15,
-  PGM_WIN32_MESSAGE       = 16
+  PGM_STATE               = 15,
+  PGM_DELETE              = 16,
+  PGM_WIN32_MESSAGE       = 17
 } PgmEventType;
 
 /**
@@ -136,6 +139,16 @@ typedef enum {
   PGM_SCROLL_DOWN
 } PgmScrollDirection;
 
+/**
+ * PgmViewportState:
+ * @PGM_VIEWPORT_ICONIFIED: The viewport is minimized.
+ *
+ * Specifies the state of a viewport.
+ */
+typedef enum {
+  PGM_VIEWPORT_ICONIFIED  = (1 << 0)
+} PgmViewportState;
+
 /**
  * PgmEventAny:
  * @type: the type of the event.
@@ -288,6 +301,23 @@ struct _PgmEventDnd {
   gchar        **uri;
 };
 
+/**
+ * PgmEventState:
+ * @type: the type of the event.
+ * @source: the source field that can be filled by applications to indicate the
+ * source of the event (mouse, pen, remote control, etc).
+ * @changed_mask: mask specifying what viewport flags have changed.
+ * @state_mask: mask specifying the new viewport flags.
+ *
+ * Generated when the state of a viewport changes.
+ */
+struct _PgmEventState {
+  PgmEventType     type;
+  guint8           source;
+  PgmViewportState changed_mask;
+  PgmViewportState state_mask;
+};
+
 /**
  * PgmEventWin32Message:
  * @type: the type of the event.
@@ -360,6 +390,7 @@ union _PgmEvent {
   PgmEventExpose       expose;
   PgmEventConfigure    configure;
   PgmEventDnd          dnd;
+  PgmEventState        state;
   PgmEventWin32Message win32_message;
 };
 
