@@ -55,7 +55,7 @@ G_BEGIN_DECLS
  * PgmViewportCursor:
  * @PGM_VIEWPORT_LEFT_ARROW: Standard left arrow system cursor.
  * @PGM_VIEWPORT_INHERIT: Inherited cursor from the environment.
- * @PGM_VIEWPORT_NONE: Hided cursor.
+ * @PGM_VIEWPORT_NONE: Hidden cursor.
  *
  * Pigment system cursor type.
  */
@@ -138,6 +138,7 @@ struct __PgmIOWatch {
  * @fullscreen: the fullscreen state.
  * @visible: the visible state.
  * @decorated: the decorated state.
+ * @iconified: the iconified state.
  * @opacity: the opacity.
  * @alpha_blending: the alpha blending state.
  * @icon: the current icon.
@@ -175,6 +176,7 @@ struct _PgmViewport {
   gboolean           fullscreen;
   gboolean           visible;
   gboolean           decorated;
+  gboolean           iconified;
   guchar             opacity;
   gboolean           alpha_blending;
 
@@ -236,6 +238,7 @@ struct _PgmViewport {
  * @drag_motion_event: the "drag-motion-event" signal.
  * @drag_drop_event: the "drag-drop-event" signal.
  * @drag_leave_event: the "drag-leave-event" signal.
+ * @state_event: the "state-event" signal.
  * @win32_message_event: the "win32-message-event" signal.
  * @pixels_read: the "pixels-read" signal.
  * @update_pass: the "update-pass" signal.
@@ -250,7 +253,8 @@ struct _PgmViewport {
  * @set_alpha_blending: the set_alpha_blending virtual method.
  * @set_opacity: the set_opacity virtual method.
  * @set_fullscreen: the set_fullscreen virtual method.
- * @get_fullscreen: the get_fullscreen virtual method.
+ * @set_iconified: the set_iconified virtual method.
+ * @focus: the focus virtual method.
  * @set_screen_resolution: the set_screen_resolution virtual method.
  * @get_screen_resolution: the get_screen_resolution virtual method.
  * @set_screen_size_mm: the set_screen_size_mm virtual method.
@@ -313,6 +317,9 @@ struct _PgmViewportClass {
   void     (*drag_leave_event)      (PgmViewport *viewport,
                                      PgmEventDnd *event);
 
+  void     (*state_event)           (PgmViewport *viewport,
+                                     PgmEventState *event);
+
   void     (*win32_message_event)   (PgmViewport *viewport,
                                      PgmEventWin32Message *event);
 
@@ -355,8 +362,11 @@ struct _PgmViewportClass {
 
   PgmError (*set_fullscreen)        (PgmViewport *viewport,
                                      gboolean fullscreen);
-  PgmError (*get_fullscreen)        (PgmViewport *viewport,
-                                     gboolean *fullscreen);
+
+  PgmError (*set_iconified)         (PgmViewport *viewport,
+                                     gboolean iconified);
+
+  PgmError (*focus)                 (PgmViewport *viewport);
 
   PgmError (*set_screen_resolution) (PgmViewport *viewport,
                                      gint width,
@@ -456,6 +466,13 @@ PgmError     pgm_viewport_set_fullscreen        (PgmViewport *viewport,
 PgmError     pgm_viewport_get_fullscreen        (PgmViewport *viewport,
                                                  gboolean *fullscreen);
 
+PgmError     pgm_viewport_set_iconified         (PgmViewport *viewport,
+                                                 gboolean iconified);
+PgmError     pgm_viewport_get_iconified         (PgmViewport *viewport,
+                                                 gboolean *iconified);
+
+PgmError     pgm_viewport_focus                 (PgmViewport *viewport);
+
 PgmError     pgm_viewport_set_screen_resolution (PgmViewport *viewport,
                                                  gint width,
                                                  gint height);
