@@ -912,6 +912,45 @@ do_free_texture (PgmContext *context,
   pgm_texture_free (PGM_TEXTURE (data));
 }
 
+#ifdef HAVE_WGL_H
+/* On Windows with the Intel 945GM driver, scene rendered with only one texture
+ * with texture data uploaded to video memory are very very slow. Seems like a
+ * bug in the Intel driver. The workaround is to draw an invisible texture
+ * mapped triangle with that card on Windows. */
+static void
+do_intel945gm_black_magic (PgmContext *context)
+{
+  static PgmGlUint black_magic_tex_id = 0;
+  static PgmGlFloat vertex[9] = { 0.0f };
+  static PgmGlFloat color[9] = { 0.0f };
+  PgmContextProcAddress *gl = context->gl;
+
+  /* Create invisible texture */
+  if (G_UNLIKELY (black_magic_tex_id == 0))
+    {
+      gl->gen_textures (1, &black_magic_tex_id);
+      gl->bind_texture (PGM_GL_TEXTURE_2D, black_magic_tex_id);
+      gl->tex_image_2d (PGM_GL_TEXTURE_2D, 0, PGM_GL_RGBA, 8, 8, 0,
+                        PGM_GL_RGBA, PGM_GL_UNSIGNED_BYTE, NULL);
+      gl->tex_parameter_i (PGM_GL_TEXTURE_2D, PGM_GL_TEXTURE_MIN_FILTER,
+                           PGM_GL_NEAREST);
+      gl->tex_parameter_i (PGM_GL_TEXTURE_2D, PGM_GL_TEXTURE_MAG_FILTER,
+                           PGM_GL_NEAREST);
+    }
+
+  /* Draw invisible texture mapped triangle */
+  gl->enable_client_state (PGM_GL_VERTEX_ARRAY);
+  gl->enable_client_state (PGM_GL_COLOR_ARRAY);
+  gl->vertex_pointer (3, PGM_GL_FLOAT, 0, vertex);
+  gl->color_pointer (3, PGM_GL_FLOAT, 0, color);
+  gl->bind_texture (PGM_GL_TEXTURE_2D, black_magic_tex_id);
+  gl->draw_arrays (PGM_GL_TRIANGLES, 0, 3);
+  gl->bind_texture (PGM_GL_TEXTURE_2D, 0);
+  gl->disable_client_state (PGM_GL_VERTEX_ARRAY);
+  gl->disable_client_state (PGM_GL_COLOR_ARRAY);
+}
+#endif /* HAVE_WGL_H */
+
 /* Traverse the layers and draw the drawables */
 static void
 render (PgmContext *context)
@@ -1085,6 +1124,15 @@ update_cb (gpointer data)
 
   /* Render the scene */
   render (context);
+
+#ifdef HAVE_WGL_H
+  /* The driver for Intel 945GM on Windows needs black magic. See the
+   * do_intel945gm_black_magic function documentation. */
+  if (context->intel945gm_chipset)
+    do_intel945gm_black_magic (context);
+#endif /* HAVE_WGL_H */
+
+  /* Wait for VSync and swap buffers */
   pgm_backend_wait_for_vblank (context->backend);
   pgm_backend_swap_buffers (context->backend);
   context->gl->clear (PGM_GL_COLOR_BUFFER_BIT);
@@ -1275,6 +1323,16 @@ render_loop (gpointer data)
       i++;
     }
 
+#ifdef HAVE_WGL_H
+  /* Windows driver detection for Intel 945GM chipset */
+  if (!strncmp ("Intel 945GM", context->renderer, 11))
+    {
+      GST_DEBUG_OBJECT (context, "Setting up the texture workaround for Intel "
+                        "945GM chipsets");
+      context->intel945gm_chipset = TRUE;
+  }
+#endif /* HAVE_GL_WGL_H */
+
   /* Check in the environment if we have to disable ARB vp/fp */
   if (context->feature_mask & PGM_GL_FEAT_FRAGMENT_PROGRAM)
     {
@@ -1433,6 +1491,11 @@ init_context (PgmContext *context)
   g_get_current_time (&context->fps_tick_time);
   context->fps = 0;
 
+#ifdef HAVE_WGL_H
+  /* Windows driver for Intel 945GM chipset */
+  context->intel945gm_chipset = FALSE;
+#endif /* HAVE_GL_WGL_H */
+
   /* Create rendering thread */
   context->render_thread = g_thread_create (render_loop, context, TRUE, &error);
   if (error != NULL)
