@@ -149,6 +149,10 @@ struct _PgmContext {
 
   /* Task handlers */
   PgmContextTaskFunc task_func[PGM_CONTEXT_LAST_TASK];
+
+#ifdef HAVE_WGL_H
+  gboolean intel945gm_chipset;
+#endif /* HAVE_GL_WGL_H */
 };
 
 /* Task */
