@@ -415,7 +415,7 @@ load_glx_extensions (PgmGlxBackend *glxbackend)
  *
  * An ARGB visual could be searched on demand with the "PGM_GL_ARGB_VISUAL"
  * environment variable. This is sad but ARGB visuals are not correctly
- *  supported by the DRI (the issue will be fixed with redirected direct
+ * supported by the DRI (the issue will be fixed with redirected direct
  * rendering and DRI2) and with the NVIDIA proprietary driver there were last
  * time I tried vertical synchronization issues.
  */
@@ -1095,6 +1095,29 @@ handle_xdnd_leave_event (PgmGlxBackend *glxbackend,
   return (PgmEvent*) pgmevent;
 }
 
+/* Send a focus request */
+static void
+send_focus_message (PgmGlxBackend *glxbackend)
+{
+  XClientMessageEvent xclient;
+
+  memset (&xclient, 0, sizeof (xclient));
+  xclient.type = ClientMessage;
+  xclient.window = glxbackend->win;
+  xclient.message_type = glxbackend->atom[ATOM_NET_ACTIVE_WINDOW];
+  xclient.format = 32;
+  xclient.data.l[0] = 1;
+  xclient.data.l[1] = CurrentTime;
+  xclient.data.l[2] = None;
+  xclient.data.l[3] = 0;
+  xclient.data.l[4] = 0;
+
+  XSendEvent (glxbackend->dpy, glxbackend->root, False,
+              SubstructureRedirectMask | SubstructureNotifyMask,
+              (XEvent*) &xclient);
+  XSync (glxbackend->dpy, False);
+}
+
 /* Escape string for X messages */
 static gchar*
 escape_for_x_message (const gchar *str)
@@ -1388,6 +1411,22 @@ create_button_event (PgmGlxBackend *glxbackend,
   return event;
 }
 
+/* Build and push a viewport state event */
+static void
+push_state_event (PgmBackend *backend,
+                  PgmViewportState changed_mask,
+                  PgmViewportState state_mask)
+{
+  PgmGlViewport *glviewport = backend->context->glviewport;
+  PgmViewport *viewport = PGM_VIEWPORT (glviewport);
+  PgmEventState *event;
+
+  event = (PgmEventState*) pgm_event_new (PGM_STATE);
+  event->changed_mask = changed_mask;
+  event->state_mask = state_mask;
+  pgm_viewport_push_event (viewport, (PgmEvent*) event);
+}
+
 /* Dispatches events pushing them on the GlViewport queue */
 static gboolean
 dispatch_x_events (PgmBackend *backend)
@@ -1543,6 +1582,30 @@ dispatch_x_events (PgmBackend *backend)
           pgm_viewport_push_event (viewport, pgmevent);
           break;
 
+          /* --- Map notify event --- */
+        case MapNotify:
+          GST_DEBUG_OBJECT (glxbackend, "MapNotify event");
+
+          if (glxbackend->state & PGM_VIEWPORT_ICONIFIED)
+            {
+              glxbackend->state &= ~PGM_VIEWPORT_ICONIFIED;
+              push_state_event (backend, PGM_VIEWPORT_ICONIFIED,
+                                glxbackend->state);
+            }
+          break;
+
+          /* --- Unmap notify event --- */
+        case UnmapNotify:
+          GST_DEBUG_OBJECT (glxbackend, "UnmapNotify event");
+
+          if (! (glxbackend->state & PGM_VIEWPORT_ICONIFIED))
+            {
+              glxbackend->state |= PGM_VIEWPORT_ICONIFIED;
+              push_state_event (backend, PGM_VIEWPORT_ICONIFIED,
+                                glxbackend->state);
+            }
+          break;
+
           /* --- Selection notify event --- */
         case SelectionNotify:
           GST_DEBUG_OBJECT (glxbackend, "SelectionNotify event: "
@@ -1893,6 +1956,7 @@ pgm_glx_backend_set_visibility (PgmBackend *backend,
   if (visible)
     {
       XMapWindow (glxbackend->dpy, glxbackend->win);
+      glxbackend->mapped = TRUE;
 
       /* If the fullscreen state has changed to FALSE during the hidden period */
 
@@ -1911,17 +1975,65 @@ pgm_glx_backend_set_visibility (PgmBackend *backend,
     {
       XUnmapWindow (glxbackend->dpy, glxbackend->win);
       XSync (glxbackend->dpy, False);
+      glxbackend->mapped = FALSE;
     }
 
   return TRUE;
 }
 
+static gboolean
+pgm_glx_backend_set_iconified (PgmBackend *backend,
+                               gboolean iconified)
+{
+  PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
+
+  GST_DEBUG_OBJECT (glxbackend, "set_iconified");
+
+  if (iconified)
+    {
+      if (glxbackend->mapped)
+        {
+          XIconifyWindow (glxbackend->dpy, glxbackend->win, glxbackend->screen);
+          XSync (glxbackend->dpy, False);
+        }
+      else
+        {
+          glxbackend->state |= PGM_VIEWPORT_ICONIFIED;
+          push_state_event (backend, PGM_VIEWPORT_ICONIFIED, glxbackend->state);
+        }
+    }
+  else
+    {
+      if (glxbackend->mapped)
+        send_focus_message (glxbackend);
+      else
+        {
+          glxbackend->state &= ~PGM_VIEWPORT_ICONIFIED;
+          push_state_event (backend, PGM_VIEWPORT_ICONIFIED, glxbackend->state);
+        }
+    }
+
+  return TRUE;
+}
+
+static void
+pgm_glx_backend_focus (PgmBackend *backend)
+{
+  PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
+
+  GST_LOG_OBJECT (glxbackend, "focus");
+
+  if (!glxbackend->mapped)
+    pgm_glx_backend_set_visibility (backend, TRUE);
+
+  send_focus_message (glxbackend);
+}
+
 static gboolean
 pgm_glx_backend_set_decorated (PgmBackend *backend,
                                gboolean decorated)
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
-  XClientMessageEvent xclient;
   MwmHints *hints, new_hints;
   gulong nitems, bytes_after;
   guchar *data;
@@ -1964,20 +2076,7 @@ pgm_glx_backend_set_decorated (PgmBackend *backend,
 
   /* Send a focus request message so that the window keeps the focus after
    * the decorations change */
-  memset (&xclient, 0, sizeof (xclient));
-  xclient.type = ClientMessage;
-  xclient.window = glxbackend->win;
-  xclient.message_type = glxbackend->atom[ATOM_NET_ACTIVE_WINDOW];
-  xclient.format = 32;
-  xclient.data.l[0] = 1;
-  xclient.data.l[1] = CurrentTime;
-  xclient.data.l[2] = None;
-  xclient.data.l[3] = 0;
-  xclient.data.l[4] = 0;
-  XSendEvent (glxbackend->dpy, glxbackend->root, False,
-              SubstructureRedirectMask | SubstructureNotifyMask,
-              (XEvent*) &xclient);
-  XSync (glxbackend->dpy, False);
+  send_focus_message (glxbackend);
 
   return TRUE;
 }
@@ -2559,6 +2658,9 @@ pgm_glx_backend_class_init (PgmGlxBackendClass *klass)
     GST_DEBUG_FUNCPTR (pgm_glx_backend_set_fullscreen);
   backend_class->set_visibility =
     GST_DEBUG_FUNCPTR (pgm_glx_backend_set_visibility);
+  backend_class->set_iconified =
+    GST_DEBUG_FUNCPTR (pgm_glx_backend_set_iconified);
+  backend_class->focus = GST_DEBUG_FUNCPTR (pgm_glx_backend_focus);
   backend_class->get_screen_size_mm =
     GST_DEBUG_FUNCPTR (pgm_glx_backend_get_screen_size_mm);
   backend_class->set_screen_resolution =
@@ -2625,6 +2727,8 @@ pgm_glx_backend_init (PgmGlxBackend *glxbackend)
   glxbackend->drag_uri = NULL;
   glxbackend->created = FALSE;
   glxbackend->fullscreen = FALSE;
+  glxbackend->mapped = FALSE;
+  glxbackend->state = 0;
   glxbackend->windowed_width = 800;
   glxbackend->windowed_height = 600;
   glxbackend->vendor = 0;
